@@ -17,7 +17,7 @@ import requests
 import bz2#bz2file pypi
 import zipfile
 
-def _download_file_from_google_drive(id, destination):
+def download_file_from_google_drive(id, destination):
     URL = "https://docs.google.com/uc?export=download"
     session = requests.Session()
     response = session.get(URL, params = { 'id' : id }, stream = True)
@@ -53,11 +53,14 @@ def _bz2Decode(filepath):
     open(newfilepath, 'wb').write(data) # write a uncompressed file
 def downloadImageSamples(folder='train'):
     '''
+    downloadImageSamples(folder='train')
     download sample images to 'folder'
     '''
+    #vedio==> https://drive.google.com/file/d/1076Ftdz8hxZkly-7QxYUR6kRJHSft-7s/view?usp=sharing
+    #images==> https://drive.google.com/file/d/1zmpZY5D5vNwcxhNmTgBprisevixexA4I/view
     file_id = '1zmpZY5D5vNwcxhNmTgBprisevixexA4I'
     destination = 'train.zip'
-    _download_file_from_google_drive(file_id, destination)
+    download_file_from_google_drive(file_id, destination)
     with zipfile.ZipFile(destination, mode='r') as myzip:
         for file in myzip.namelist():
             print("extract "+file)
@@ -65,6 +68,12 @@ def downloadImageSamples(folder='train'):
     os.remove(destination)
     
 def downloadFileInNeed(filename):
+'''
+downloadFileInNeed(filename)
+dowload file from dlib web site:
+http://dlib.net/files
+filename: for example ==>"shape_predictor_68_face_landmarks.dat"
+'''
     if not os.path.isfile(filename):
         print("載入"+filename+"...可能需要一點時間") 
         url="http://dlib.net/files/"+filename+".bz2"
@@ -72,6 +81,7 @@ def downloadFileInNeed(filename):
         _bz2Decode(filename+".bz2")
         os.remove(filename+".bz2")
         print("ok")
+        
 downloadFileInNeed("shape_predictor_68_face_landmarks.dat")  
 downloadFileInNeed("dlib_face_recognition_resnet_model_v1.dat")
     
@@ -133,12 +143,12 @@ def loadDB(db=DB,folder='train',isClearDB=True):
     
     the file name would be the TAG name
     '''
-    if isClearDB:DB.clear()
+    if isClearDB:db.clear()
     for f in os.listdir(folder):
         path = folder+"\\"+f
         if os.path.isfile(path):
             img = io.imread(path)
-            DB[os.path.splitext(f)[0]]=getFeatureVector(img)
+            db[os.path.splitext(f)[0]]=getFeatureVector(img)
             print(f+" feature loaded")
 
 def loadFeatureFromPic(imagePath):
@@ -200,17 +210,20 @@ def addText2Img_cv2(img_cv2,text,font=_FONT,position=(20,20),fill=(255,0,0)):
     img_PIL.close()
     return img_cv2
 
-def captureImageFromCam():
+def captureImageFromCam(camSerial=None):
     '''
-    captureImageFromCam()
+    captureImageFromCam(camSerial=None)
     return cv2 image capture from webcam
     '''
     ret = None
     cap = None
-    for i in range(3):#assume there are less than 3 webcam attached to the platform
-        cap = cv2.VideoCapture(i)
-        if cap.isOpened(): break   
+    if not camSerial:
+        for i in range(5):#assume there are less than 5 webcam attached to the platform
+            cap = cv2.VideoCapture(i)
+            if cap.isOpened():break
+    else: cap = cv2.VideoCapture(camSerial)
     if not cap:return None
+    
     # 載入人臉檢測器
     while(True):
         if cap.isOpened():
@@ -221,23 +234,26 @@ def captureImageFromCam():
     return ret
 
 #由webcam擷取訓練影像 and save          
-def capturePicFromCamAndSave(tag,folder="train"):
+def captureImageFromCamAndSave(tag,folder="train",camSerial=None):
     '''
-    capturePicFromCamAndSave(tag,folder="train")
+    captureImageFromCamAndSave(tag,folder="train",camSerial=None)
     capture picture from camera and save it as:
     folder/tag.jpg
     return cv2 image
     '''
-    ret = False
+    ret = None
     cap = None
-    for i in range(3):#assume there are less than 3 webcam attached to the platform
-        cap = cv2.VideoCapture(i)
-        if cap.isOpened(): break
-        
-    if not cap:return False
+    if not camSerial:
+        for i in range(5):#assume there are less than 5 webcam attached to the platform
+            cap = cv2.VideoCapture(i)
+            if cap.isOpened():break
+    else: cap = cv2.VideoCapture(camSerial)
+    if not cap:return None
+    
     cv2.startWindowThread()
     # 載入人臉檢測器
     text="press p to take picture"
+    # 先建立照片存放之資料夾(預設為train)
     _createFolder(folder)
 
     while(True):
@@ -249,9 +265,10 @@ def capturePicFromCamAndSave(tag,folder="train"):
                     rect=_detector(frame, 1)[0] #取第一個檢測到的臉區域# IndexError
                     if cv2.waitKey(1) & 0xFF == ord('p'):
                         im=cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)
+                        
                         io.imsave(folder+"\\"+tag+".jpg",im)
                         text=" pictures saved..."
-                        ret = True
+                        ret = frame
                         break
                     cv2.rectangle(frame,(rect.left(),rect.top()),(rect.right(),rect.bottom()),(255,0,0),3)
                     cv2.putText(frame, text,(rect.left()-80, rect.top()-20), cv2.FONT_HERSHEY_SIMPLEX,1, (255, 0, 0), 2, cv2.LINE_AA)    
@@ -265,9 +282,55 @@ def capturePicFromCamAndSave(tag,folder="train"):
     cv2.destroyAllWindows()
     return ret
 
+def predictCam(camSerial=None,skipFranmes=1,outputFileName=""):
+    '''
+        predictCam(camSerial=None,skipFranmes=1,outputFileName="")
+        
+        perform predict on webcam the show the result to window
+        press esc to exit
+        
+        skipFranmes: perform prediction every <skipFrames> frames
+        outputFilename:output the predicted result(tag) to file<outputFilename>
+    '''
+    #即時辨識
+    cap = None
+    if not camSerial:
+        for i in range(5):
+            cap = cv2.VideoCapture(i)
+            if cap.isOpened():break
+    else: cap = cv2.VideoCapture(camSerial)
+    cv2.startWindowThread()
+    _detector = dlib.get_frontal_face_detector()
+    count = 0
+    while(True):    
+        if not cap.isOpened():break
+        ret, frame = cap.read()
+        count+=1
+        if count%skipFranmes!=0 :continue
+        if ret: 
+            try:
+                rects=_detector(frame, 1)
+                for rect in rects:
+                    V=getFeatureVector(frame,rect)
+                    Tag,dist=predictFromDB(V) # predict the target Tag
+                    cv2.rectangle(frame,(rect.left(),rect.top()),(rect.right(),rect.bottom()),(255,0,0),3)
+                    #cv2.putText(frame, Tag+":"+str(dist),, cv2.FONT_HERSHEY_SIMPLEX,1, (255, 0, 0), 2, cv2.LINE_AA)
+                    text=Tag+":"+str(dist)
+                    frame=addText2Img_cv2(frame,Tag+":"+str(round(dist,3)),_FONT,(rect.left(), rect.top()-_FONT.size-1))        
+                    if outputFileName!="":
+                        with open(outputFileName,'w') as f:
+                            f.write(Tag)
+            except IndexError:pass    
+            cv2.imshow('press esc to exit...', frame)
+        if cv2.waitKey(10) == 27: break
+    cap.release()
+    cv2.destroyAllWindows()
+
 def predictVedio(vedioPath,skipFranmes=10,db=DB):
     '''
     predictVedio(vedioPath,skipFranmes=10,db=DB)
+    skipFranmes: perform prediction every <skipFrames> frames
+    
     demo predicion from the vedio file
     press esc to exit
     '''
@@ -294,45 +357,9 @@ def predictVedio(vedioPath,skipFranmes=10,db=DB):
     cap.release()
     cv2.destroyAllWindows()
     
-
-def predictCam(camSerial=None,outputFileName=""):
-    '''
-        predictCam(camSerial=None,outputFileName="")
-        perform predict on webcam the show the result to window
-        press esc to exit
-    '''
-    #即時辨識
-    cap = None
-    if not camSerial:
-        for i in range(5):
-            cap = cv2.VideoCapture(i)
-            if cap.isOpened():break
-    else: cap = cv2.VideoCapture(camSerial)
-    cv2.startWindowThread()
-    _detector = dlib.get_frontal_face_detector()
-    while(True):    
-        if not cap.isOpened():break
-        ret, frame = cap.read()
-        if ret: 
-            try:
-                rects=_detector(frame, 1)
-                for rect in rects:
-                    V=getFeatureVector(frame,rect)
-                    Tag,dist=predictFromDB(V) # predict the target Tag
-                    cv2.rectangle(frame,(rect.left(),rect.top()),(rect.right(),rect.bottom()),(255,0,0),3)                    #cv2.putText(frame, Tag+":"+str(dist),, cv2.FONT_HERSHEY_SIMPLEX,1, (255, 0, 0), 2, cv2.LINE_AA)
-                    text=Tag+":"+str(dist)
-                    frame=addText2Img_cv2(frame,Tag+":"+str(round(dist,3)),_FONT,(rect.left(), rect.top()-_FONT.size-1))        
-                    if outputFileName!="":
-                        with open(outputFileName,'w') as f:
-                            f.write(Tag)
-            except IndexError:pass    
-            cv2.imshow('press esc to exit...', frame)
-        if cv2.waitKey(10) == 27: break
-    cap.release()
-    cv2.destroyAllWindows()
-
-
+    
 if __name__ == "__main__":
+    #captureImageFromCamAndSave('李國源',camSerial=1)
     '''
     #download the samples to 'train' folder
     downloadImageSamples()
@@ -352,3 +379,6 @@ if __name__ == "__main__":
     loadDB(folder='train')
     predictCam()
     '''
+    loadDB(folder='train')
+    predictCam(camSerial=1,outputFileName="result.txt")
+    
\ No newline at end of file
